@@ -23,51 +23,34 @@ import (
 	"sort"
 
 	"github.com/bishopfox/sliver/client/assets"
-
-	"gopkg.in/AlecAivazis/survey.v1"
+	"github.com/bishopfox/sliver/client/forms"
 )
 
-func selectConfig() *assets.ClientConfig {
-
+func selectConfig() (string, *assets.ClientConfig) {
 	configs := assets.GetConfigs()
 
 	if len(configs) == 0 {
-		return nil
+		return "", nil
 	}
 
 	if len(configs) == 1 {
-		for _, config := range configs {
-			return config
+		for key, config := range configs {
+			return key, config
 		}
 	}
 
-	answer := struct{ Config string }{}
-	qs := getPromptForConfigs(configs)
-	err := survey.Ask(qs, &answer)
-	if err != nil {
-		fmt.Println(err.Error())
-		return nil
-	}
-
-	return configs[answer.Config]
-}
-
-func getPromptForConfigs(configs map[string]*assets.ClientConfig) []*survey.Question {
-
-	keys := []string{}
-	for k := range configs {
-		keys = append(keys, k)
+	keys := make([]string, 0, len(configs))
+	for key := range configs {
+		keys = append(keys, key)
 	}
 	sort.Strings(keys)
 
-	return []*survey.Question{
-		{
-			Name: "config",
-			Prompt: &survey.Select{
-				Message: "Select a server:",
-				Options: keys,
-				Default: keys[0],
-			},
-		},
+	selection := keys[0]
+	err := forms.Select("Select a server:", keys, &selection)
+	if err != nil {
+		fmt.Println(err.Error())
+		return "", nil
 	}
+
+	return selection, configs[selection]
 }
