@@ -23,25 +23,35 @@ import (
 	"os"
 
 	"github.com/bishopfox/sliver/client/assets"
+	"github.com/rsteube/carapace"
 	"github.com/spf13/cobra"
 )
 
-var cmdImport = &cobra.Command{
-	Use:   "import",
-	Short: "Import a client configuration file",
-	Long:  `import [config files]`,
-	Run: func(cmd *cobra.Command, args []string) {
-		if 0 < len(args) {
-			for _, arg := range args {
-				conf, err := assets.ReadConfig(arg)
-				if err != nil {
-					fmt.Printf("[!] %s\n", err)
-					os.Exit(3)
+func importCmd() *cobra.Command {
+	cmdImport := &cobra.Command{
+		Use:   "import",
+		Short: "Import a client configuration file",
+		Long:  `import [config files]`,
+		Run: func(cmd *cobra.Command, args []string) {
+			if 0 < len(args) {
+				for _, arg := range args {
+					conf, err := assets.ReadConfig(arg)
+					if err != nil {
+						fmt.Printf("[!] %s\n", err)
+						os.Exit(3)
+					}
+					assets.SaveConfig(conf)
 				}
-				assets.SaveConfig(conf)
+			} else {
+				fmt.Printf("Missing config file path, see --help")
 			}
-		} else {
-			fmt.Printf("Missing config file path, see --help")
-		}
-	},
+		},
+		ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+			return []string{}, cobra.ShellCompDirectiveDefault
+		},
+	}
+
+	carapace.Gen(cmdImport).PositionalCompletion(carapace.ActionFiles().Tag("server configuration"))
+
+	return cmdImport
 }
