@@ -4,15 +4,16 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/AlecAivazis/survey/v2"
 	"github.com/bishopfox/sliver/client/console"
+	"github.com/bishopfox/sliver/client/forms"
 	"github.com/bishopfox/sliver/protobuf/clientpb"
-	"github.com/desertbit/grumble"
+	"github.com/spf13/cobra"
 )
 
-// ImplantsRmCmd - Deletes an archived implant build from the server
-func ImplantsRmCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
-	name := ctx.Args.String("name")
+// ImplantsRmCmd - Deletes an archived implant build from the server.
+func ImplantsRmCmd(cmd *cobra.Command, con *console.SliverClient, args []string) {
+	name := args[0]
+	// name := ctx.Args.String("name")
 	if name == "" {
 		con.PrintErrorf("No name specified\n")
 		return
@@ -23,8 +24,7 @@ func ImplantsRmCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
 		return
 	}
 	confirm := false
-	prompt := &survey.Confirm{Message: fmt.Sprintf("Remove '%s' build?", name)}
-	survey.AskOne(prompt, &confirm)
+	_ = forms.Confirm(fmt.Sprintf("Remove '%s' build?", name), &confirm)
 	if !confirm {
 		return
 	}
