@@ -21,13 +21,14 @@ package monitor
 import (
 	"context"
 
+	"github.com/spf13/cobra"
+
 	"github.com/bishopfox/sliver/client/console"
 	"github.com/bishopfox/sliver/protobuf/commonpb"
-	"github.com/desertbit/grumble"
 )
 
 // MonitorStartCmd - Start monitoring threat intel for implants
-func MonitorStartCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
+func MonitorStartCmd(cmd *cobra.Command, con *console.SliverClient, args []string) {
 	resp, err := con.Rpc.MonitorStart(context.Background(), &commonpb.Empty{})
 	if err != nil {
 		con.PrintErrorf("%s\n", err)
