@@ -22,19 +22,18 @@ import (
 	"fmt"
 	"os"
 
-	"github.com/AlecAivazis/survey/v2"
 	"github.com/bishopfox/sliver/client/console"
 	"github.com/bishopfox/sliver/client/core"
-	"github.com/desertbit/grumble"
+	"github.com/bishopfox/sliver/client/forms"
+	"github.com/spf13/cobra"
 )
 
-// ReactionSaveCmd - Manage reactions to events
-func ReactionSaveCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
+// ReactionSaveCmd - Manage reactions to events.
+func ReactionSaveCmd(cmd *cobra.Command, con *console.SliverClient, args []string) {
 	reactionPath := GetReactionFilePath()
 	if _, err := os.Stat(reactionPath); !os.IsNotExist(err) {
 		confirm := false
-		prompt := &survey.Confirm{Message: fmt.Sprintf("Overwrite reactions (%s) on disk?", reactionPath)}
-		survey.AskOne(prompt, &confirm)
+		_ = forms.Confirm(fmt.Sprintf("Overwrite reactions (%s) on disk?", reactionPath), &confirm)
 		if !confirm {
 			return
 		}
