@@ -22,13 +22,13 @@ import (
 	"context"
 
 	"github.com/bishopfox/sliver/client/console"
+	"github.com/bishopfox/sliver/client/core"
 	"github.com/bishopfox/sliver/protobuf/sliverpb"
-	"github.com/desertbit/grumble"
+	"github.com/spf13/cobra"
 )
 
-// CloseSessionCmd - Close an interactive session but do not kill the remote process
-func CloseSessionCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
-
+// CloseSessionCmd - Close an interactive session but do not kill the remote process.
+func CloseSessionCmd(cmd *cobra.Command, con *console.SliverClient, args []string) {
 	// Get the active session
 	session := con.ActiveTarget.GetSessionInteractive()
 	if session == nil {
@@ -36,9 +36,19 @@ func CloseSessionCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
 		return
 	}
 
+	// remove any active socks proxies
+	socks := core.SocksProxies.List()
+	if len(socks) != 0 {
+		for _, p := range socks {
+			if p.SessionID == session.ID {
+				core.SocksProxies.Remove(p.ID)
+			}
+		}
+	}
+
 	// Close the session
 	_, err := con.Rpc.CloseSession(context.Background(), &sliverpb.CloseSession{
-		Request: con.ActiveTarget.Request(ctx),
+		Request: con.ActiveTarget.Request(cmd),
 	})
 	if err != nil {
 		con.PrintErrorf("%s\n", err.Error())
@@ -46,5 +56,4 @@ func CloseSessionCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
 	}
 
 	con.ActiveTarget.Set(nil, nil)
-
 }
