@@ -5,17 +5,20 @@ import (
 
 	"github.com/bishopfox/sliver/client/console"
 	"github.com/bishopfox/sliver/protobuf/clientpb"
-	"github.com/desertbit/grumble"
+	"github.com/spf13/cobra"
 )
 
-// TasksCancelCmd - Cancel a beacon task before it's sent to the implant
-func TasksCancelCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
+// TasksCancelCmd - Cancel a beacon task before it's sent to the implant.
+func TasksCancelCmd(cmd *cobra.Command, con *console.SliverClient, args []string) {
 	beacon := con.ActiveTarget.GetBeaconInteractive()
 	if beacon == nil {
 		return
 	}
 
-	idArg := ctx.Args.String("id")
+	var idArg string
+	if len(args) > 0 {
+		idArg = args[0]
+	}
 	var task *clientpb.BeaconTask
 	var err error
 	if idArg == "" {
@@ -47,6 +50,9 @@ func TasksCancelCmd(ctx *grumble.Context, con *console.SliverConsoleClient) {
 			con.PrintErrorf("%s\n", err)
 			return
 		}
+		// Request and response content is not needed for cancelling a task
+		task.Request = nil
+		task.Response = nil
 	}
 
 	if task != nil {
